'use strict';

const util = require('util')
const rename = {
	'client_unique_identifier': 'uid',
	'client_database_id': 'dbid',
	'client_nickname': 'name',
	'client_status': 'status',
	'clid': 'clid',
	'cid': 'cid',
	'invokerid': 'clid',
	'invokername': 'name',
	'invokeruid': 'uid'
};

let alfred;

class User {

	/**
	 * Wrapper to ease the process of interacting with users
	 * 
	 * @author schroffl
	 * 
	 * @param {Object} data - Data containing information about the user
	 *
	 * @constructor
	 */

	constructor(data) {
		this.detail = {};

		for(const prop in data) {
			if(prop in rename)
				this[rename[prop]] = data[prop];
			else
				this.detail[prop] = data[prop];
		}
	}

	/**
	 * Respond to a user via 'sendtextmessage'
	 * 
	 * @author schroffl
	 * 
	 * @param {...*} message - Concatenated via whitespace
	 */

	respond() {
		const args = Array.from(arguments);

		for(let i=0; i<args.length; i++)
			if(typeof args[i] === 'object') args[i] = util.inspect(args[i]);

		alfred.send('sendtextmessage', { 'targetmode': 1, 'target': this.clid, 'msg': args.join(' ') });
	}

	/**
	 * Poke a user via 'clientpoke'
	 * 
	 * @author schroffl
	 * 
	 * @param {...*} message - Concatenated via whitespace
	 */

	poke() {
		const args = Array.from(arguments);

		for(let i=0; i<args.length; i++)
			if(typeof args[i] === 'object') args[i] = util.inspect(args[i]);

		alfred.send('clientpoke', { 'clid': this.clid, 'msg': args.join(' ') });
	}

	/**
	 * Move a user via 'clientmove'
	 * 
	 * @author schroffl
	 * 
	 * @param {Number} channelid - The channel to move the client into
	 */

	move(cid) {
		alfred.send('clientmove', { 'clid': this.clid, cid });
	}
}

module.exports = function() {
	const args = Array.from(arguments),
		  event = args.shift(),
		  next = args.pop();

	alfred = this;

	if(event !== 'login') args[0].user = new User(args[0]);

	next();
}